use std::{sync::Arc, thread, time::Duration};

use crossbeam_channel::Receiver;
use egui::{mutex::Mutex};

use crate::{config, omori_locator};

#[derive(Clone)]
pub enum UiState {
    Loading,
    KeyRequired(String),
    Error(String)
}

#[derive(Clone)]
pub struct UiStateHolder {
    pub state: Arc<Mutex<UiState>>
}

pub enum UiEvent {
    SetKey(Vec<u8>)
}

pub struct AppThread {
    ui_state: UiStateHolder,
    context: egui::Context,
    ui_event_channel: Receiver<UiEvent>,
    decryption_key: Vec<u8>
}

impl AppThread {
    fn commit(&self, s: UiState) {
        let mut state = self.ui_state.state.lock();
        *state = s;
    }

    fn get_key(&mut self, reason: String) -> anyhow::Result<()> {
        self.commit(UiState::KeyRequired(reason));
        loop {
            match self.ui_event_channel.recv()? {
                UiEvent::SetKey(key) => {
                    self.decryption_key = key;
                    break
                }
                _ => {}
            }
        }

        self.commit(UiState::Loading);

        Ok(())
    }

    pub fn create(state_holder: &UiStateHolder, context: &egui::Context, ui_event_channel: &Receiver<UiEvent>) -> AppThread {
        AppThread {
            ui_state: state_holder.clone(),
            context: context.clone(),
            ui_event_channel: ui_event_channel.clone(),
            decryption_key: Vec::new()
        }
    }

    fn real_main(&mut self) -> anyhow::Result<()> {
        self.commit(UiState::Loading);

        let mut config_provider = config::ConfigProvider::new()?;
        println!("{:?}", config_provider);

        match omori_locator::get_omori_key() {
            Ok(key) => self.decryption_key = key,
            Err(reason) => self.get_key(format!("{:?}", reason))?,
        }

        Ok(())
    }

    pub fn main(&mut self) {
        match self.real_main() {
            Ok(_) => {},
            Err(e) => self.commit(UiState::Error(format!("{:?}", e))),
        }
    }
}
